package edu.chl.hajo.monopoly.view;

import edu.chl.hajo.monopoly.core.Monopoly;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.core.Space;
import edu.chl.hajo.monopoly.event.Event;
import edu.chl.hajo.monopoly.event.IEventHandler;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

/**

 @author hajo
 */
public class MainWindow extends JPanel implements ActionListener, IEventHandler {

    private static final int width = 600;  // For game area
    private static final int height = 600;
    public final Monopoly monopoly;

    public MainWindow(Monopoly monopoly) {
        this.monopoly = monopoly;
        initGui();
        initEvent();
    }

    // This is events going DOWN to model
    @Override
    public void actionPerformed(ActionEvent e) {

    }

    // This is event comming UP from model
    @Override
    public void onEvent(Event evt) {
            // TODO 
    }


    private void initEvent() {
        // TODO 
    }

    private void initGui() {
        setPreferredSize(new Dimension(width, height));
        JFrame window = new JFrame();
        window.getRootPane().setDoubleBuffered(true);
        window.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        window.setTitle("MONOPOLY");

        // Create the GUI (composed of panels)
        this.setLayout(new BorderLayout());
        // Panels
        this.add(createDicesPanel(), BorderLayout.NORTH);
        this.add(createSpacesPanel(), BorderLayout.CENTER);
        this.add(createPlayersPanel(), BorderLayout.SOUTH);

        window.add(this);
        window.pack();
        window.setLocationRelativeTo(null);
        window.setVisible(true);
    }

    private JPanel createDicesPanel() {
        return new DicePanel(monopoly, width, 1 * height / 8);
    }

    private JPanel createSpacesPanel() {
        JPanel p = new JPanel();
        p.setPreferredSize(new Dimension(width, 5 * height / 8));
        p.setBorder(BorderFactory.createEtchedBorder());
        int side = (int) Math.sqrt(monopoly.getBoard().size());
        p.setLayout(new GridLayout(4, 10));
        for (Space s : monopoly.getBoard().getSpaces()) {
            SpacePanel sp = new SpacePanel(s.getName());         
            p.add(sp);
        }
        return p;
    }

    private JPanel createPlayersPanel() {
        JPanel p = new JPanel();
        p.setPreferredSize(new Dimension(width, 2 * height / 8));
        p.setBorder(BorderFactory.createEtchedBorder());
        for (Player player : monopoly.getPlayers()) {
            PlayerPanel pp = createPlayerPanel(player);
            p.add(pp);
        }
        return p;
    }

    private PlayerPanel createPlayerPanel(Player player) {
        PlayerPanel p = new PlayerPanel(player, (width / monopoly.getPlayers().size()) - 10,
                (2 * height / 8) - 10);
        return p;
    }

}
