package edu.chl.hajo.monopoly.core;

import edu.chl.hajo.monopoly.event.Event;
import edu.chl.hajo.monopoly.event.EventBus;
import lombok.EqualsAndHashCode;
import lombok.Getter;

/**
 A single player of the game
 Handles everything that has to do with a single player!

 @author hajo

 */
@EqualsAndHashCode( of = "name")
public class Player {

    // To avoid null as owner of IOwnables
    public static final Player NONE = new Player("none", -1);
    @Getter
    private final String name;  // Unique name for player
    //private final Piece piece;
    @Getter
    private int balance;
    private boolean inJail = false;
    @Getter
    private Space position;    // The actual position

    public Player(String name, int balance) {
        this.name = name;
        //this.piece = piece;
        this.balance = balance;
    }

    public void income(int amont) {
        setBalance(balance + amont);
    }

    public boolean isInJail() {
        return inJail;
    }

    // Event setters -----------------------
    public void setBalance(int balance) {
        this.balance = balance;
        EventBus.BUS.publish(new Event(Event.Tag.PLAYER_BALANCE, this));
    }

    public void setInJail(boolean b) {
        inJail = b;
        EventBus.BUS.publish(new Event(Event.Tag.PLAYER_IN_JAIL, this));
    }

    public void setPosition(Space position) {
        this.position = position;
        EventBus.BUS.publish(new Event(Event.Tag.PLAYER_POSITION, this));
    }

}
