package edu.chl.hajo.monopoly.core;

import java.util.ArrayList;
import java.util.List;
import static org.junit.Assert.assertTrue;
import static edu.chl.hajo.monopoly.core.TestBoard.*;
import static edu.chl.hajo.monopoly.core.TestPlayer.*;
import javax.swing.JOptionPane;
import static org.junit.Assert.fail;
import org.junit.Before;
import org.junit.Test;

/**
 Her we run integration tests (more participation objects and full use cases)

 @author hajo
 */
public class TestModel {

    private Board b;
    private List<Player> ps;

    @Before
    public void before() {
        b = new Board(getSpaces(SPACE_NAMES));
        ps = getPlayers(PLAYER_NAMES);
    }

    @Test
    public void testModelCreated() {
        Monopoly m = new Monopoly(ps, b, new Dices());
        List<Player> ps2 = m.getPlayers();
        for (Player p : ps2) {
            assertTrue(p.getPosition().equals(b.getStart()));
        }
        assertTrue(m.getActual().equals(ps.get(0)));
    }

    @Test
    public void testEndTurn() {
        // For now(?) only thing is to switch player
        final int i = 6;
        Monopoly m = new Monopoly(ps, b, new MockDices(i));
        for (Player p : m.getPlayers()) {
            m.next();
        }
        assertTrue(m.getActual().equals(ps.get(0)));
    }

    @Test
    public void testMove() {
        final int i = 6;
        Monopoly m = new Monopoly(ps, b, new MockDices(i));
        m.move();
        Player a = m.getActual();
        // No player change
        assertTrue(m.getActual().equals(ps.get(0)));
        Space s = a.getPosition();
        assertTrue(s.equals(b.getSpace(i)));
    }

    @Test
    public void testMoveAndPassGo() {
        MockDices md = new MockDices(35);
        Monopoly m = new Monopoly(ps, b, md);
        m.move();  // At Space 35
        md.setValue(10);
        m.move();
        assertTrue(m.getActual().getBalance()
                == Monopoly.BALANCE + Monopoly.BONUS);
    }
    
    @Test
    public void testMoveAndPayRent() {
        Monopoly m = new Monopoly(ps, b, new MockDices(1));
        Space s = m.getBoard().getSpace("1:1");
        Player owner = m.getPlayers().get(1);
        Street st = (Street) s;  // This is a street!
        st.setOwner(owner);  // Not actual

        m.move();

        if (m.checkPayRent()) {
            //JOptionPane.showMessageDialog(null, "Pay rent");
            m.payRent(); // TODO if no cash or broke!
        }
        assertTrue(owner.getBalance() == Monopoly.BALANCE + Board.BASIC_RENT);
        assertTrue(m.getActual().getBalance() == Monopoly.BALANCE - Board.BASIC_RENT);

    }

    @Test
    public void testMoveAndGoToJail() {
        MockDices md = new MockDices(30);  // Index of gotojail
        Monopoly m = new Monopoly(ps, b, md);
        Space gotToJail = m.getBoard().getSpace("gotojail");

        m.move();
        Player p = m.getActual();
        if (p.getPosition() == gotToJail) {
            //JOptionPane.showMessageDialog(null, "Landed on Jail");
            p.setInJail(true);
            p.setPosition(m.getBoard().getSpace("jail"));
        }
        assertTrue(p.isInJail()
                && p.getPosition().equals(m.getBoard().getSpace("jail")));
    }

    @Test
    public void testMoveCaughtSpeeding() {
        MockDices md = new MockDices(12);
        Monopoly m = new Monopoly(ps, b, md);
        m.move();  // 1. Two 6:s
        m.move();  // 2. Two 6:s
        m.move();  // 3. Two 6:s
        Player p = m.getActual();
        if (md.isSpeeding()) {
            //JOptionPane.showMessageDialog(null, "Is speeding");
            p.setInJail(true);
            p.setPosition(m.getBoard().getSpace("jail"));
        }
        assertTrue(p.isInJail()
                && p.getPosition().equals(m.getBoard().getSpace("jail")));

    }

   

}
